@@ -19,6 +19,7 @@ You should have received a copy of the GNU Lesser General Public License along w
 If not, see <https://www.gnu.org/licenses/>.
 """
 
+import gc
 import sys
 import os
 import weakref
@@ -28,12 +29,12 @@ import faulthandler
 from logging import DEBUG, INFO
 from PySide2 import QtCore, QtWidgets
 
-from qudi.core.logger import init_rotating_file_handler, init_record_model_handler
+from qudi.core.logger import init_rotating_file_handler, init_record_model_handler, clear_handlers
 from qudi.core.logger import get_logger, set_log_level
 from qudi.util.paths import get_main_dir, get_default_log_dir
 from qudi.util.mutex import Mutex
 from qudi.util.colordefs import QudiMatplotlibStyle
-from qudi.core.config import Configuration
+from qudi.core.config import Configuration, ValidationError, YAMLError
 from qudi.core.watchdog import AppWatchdog
 from qudi.core.modulemanager import ModuleManager
 from qudi.core.threadmanager import ThreadManager
@@ -49,14 +50,24 @@ try:
 except ImportError:
     pass
 
-# Set QT_API environment variable to PySide2
-os.environ['QT_API'] = 'pyside2'
+# Enable the High DPI scaling support of Qt5
+os.environ['QT_ENABLE_HIGHDPI_SCALING'] = '1'
+
+if sys.platform == 'win32':
+    # Set QT_LOGGING_RULES environment variable to suppress qt.svg related warnings that otherwise
+    # spam the log due to some known Qt5 bugs, e.g. https://bugreports.qt.io/browse/QTBUG-52079
+    os.environ['QT_LOGGING_RULES'] = 'qt.svg.warning=false'
+else:
+    # The following will prevent Qt to spam the logs on X11 systems with enough messages
+    # to significantly slow the program down. Most of those warnings should have been
+    # notice level or lower. This is a known problem since Qt does not fully comply to X11.
+    os.environ['QT_LOGGING_RULES'] = '*.debug=false;*.info=false;*.notice=false;*.warning=false'
 
 # Make icons work on non-X11 platforms, import a custom theme
 if sys.platform == 'win32':
     try:
         import ctypes
-        myappid = 'quantumoptics.qudi.mainapp'  # arbitrary string
+        myappid = 'qudicore-app'  # arbitrary string
         ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)
     except ImportError:
         raise
@@ -64,6 +75,10 @@ if sys.platform == 'win32':
         print('SetCurrentProcessExplicitAppUserModelID failed! This is probably not Microsoft '
               'Windows!')
 
+# Set default Qt locale to "C" in order to avoid surprises with number formats and other things
+# QtCore.QLocale.setDefault(QtCore.QLocale('en_US'))
+QtCore.QLocale.setDefault(QtCore.QLocale.c())
+
 
 class Qudi(QtCore.QObject):
     """
@@ -85,25 +100,50 @@ class Qudi(QtCore.QObject):
 
     def __init__(self, no_gui=False, debug=False, log_dir='', config_file=None):
         super().__init__()
+
         # CLI arguments
         self.no_gui = bool(no_gui)
         self.debug_mode = bool(debug)
         self.log_dir = str(log_dir) if os.path.isdir(log_dir) else get_default_log_dir(
             create_missing=True)
 
+        # Disable pyqtgraph "application exit workarounds" because they cause errors on exit
+        try:
+            import pyqtgraph
+            pyqtgraph.setConfigOption('exitCleanup', False)
+        except ImportError:
+            pass
+
+        # Enable stack trace output for SIGSEGV, SIGFPE, SIGABRT, SIGBUS and SIGILL signals
+        # -> e.g. for segmentation faults
+        faulthandler.disable()
+        faulthandler.enable(all_threads=True)
+
+        # install logging facility and set logging level
+        init_record_model_handler(max_records=10000)
+        init_rotating_file_handler(path=self.log_dir)
+        set_log_level(DEBUG if self.debug_mode else INFO)
+
         # Set up logger for qudi main instance
         self.log = get_logger(__class__.__name__)  # will be "qudi.Qudi" in custom logger
         sys.excepthook = self._qudi_excepthook
 
+        # Load configuration from disc if possible
+        self.configuration = Configuration()
+        try:
+            self.configuration.load(config_file, set_default=True)
+        except ValueError:
+            self.log.info('No qudi configuration file specified. Using empty default config.')
+        except (ValidationError, YAMLError):
+            self.log.exception('Invalid qudi configuration file specified. '
+                               'Falling back to default config.')
+
+        # initialize thread manager and module manager
         self.thread_manager = ThreadManager(parent=self)
         self.module_manager = ModuleManager(qudi_main=self, parent=self)
-        self.configuration = Configuration(parent=self)
-        if config_file is None:
-            config_file = Configuration.get_saved_config()
-            if config_file is None:
-                config_file = Configuration.get_default_config()
-        self.configuration.load_config(file_path=config_file, set_default=True)
-        remote_server_config = self.configuration.remote_modules_server
+
+        # initialize remote modules server if needed
+        remote_server_config = self.configuration['remote_modules_server']
         if remote_server_config:
             self.remote_modules_server = RemoteModulesServer(
                 parent=self,
@@ -117,7 +157,7 @@ class Qudi(QtCore.QObject):
                 ssl_version=remote_server_config.get('ssl_version', None),
                 cert_reqs=remote_server_config.get('cert_reqs', None),
                 ciphers=remote_server_config.get('ciphers', None),
-                force_remote_calls_by_value=self.configuration.force_remote_calls_by_value
+                force_remote_calls_by_value=self.configuration['force_remote_calls_by_value']
             )
         else:
             self.remote_modules_server = None
@@ -125,8 +165,8 @@ class Qudi(QtCore.QObject):
             parent=self,
             qudi=self,
             name='local-namespace-server',
-            port=self.configuration.namespace_server_port,
-            force_remote_calls_by_value=self.configuration.force_remote_calls_by_value
+            port=self.configuration['namespace_server_port'],
+            force_remote_calls_by_value=self.configuration['force_remote_calls_by_value']
         )
         self.watchdog = None
         self.gui = None
@@ -156,8 +196,10 @@ class Qudi(QtCore.QObject):
         for most_recent_frame, _ in traceback.walk_tb(ex_traceback):
             pass
         # Try to extract the module and class name in which the exception has been raised
+        msg = ''
         if most_recent_frame is None:
             logger = self.log
+            msg = 'Unhandled qudi exception:'
         else:
             try:
                 obj = most_recent_frame.f_locals['self']
@@ -170,8 +212,9 @@ class Qudi(QtCore.QObject):
                 except AttributeError:
                     # If no module and class name can be determined, use the application logger
                     logger = self.log
+                    msg = 'Unhandled qudi exception:'
         # Log exception with qudi log handler
-        logger.error('Unhandled qudi Exception:', exc_info=(ex_type, ex_value, ex_traceback))
+        logger.error(msg, exc_info=(ex_type, ex_value, ex_traceback))
 
     @classmethod
     def instance(cls):
@@ -192,7 +235,7 @@ class Qudi(QtCore.QObject):
                 pass
 
     def _add_extensions_to_path(self):
-        extensions = self.configuration.extension_paths
+        extensions = self.configuration['extension_paths']
         # Add qudi extension paths to sys.path
         insert_index = 1
         for ext_path in reversed(extensions):
@@ -203,8 +246,12 @@ class Qudi(QtCore.QObject):
     def _configure_qudi(self):
         """
         """
-        print(f'> Applying configuration from "{self.configuration.config_file}"...')
-        self.log.info(f'Applying configuration from "{self.configuration.config_file}"...')
+        if self.configuration.file_path is None:
+            print('> Applying default configuration...')
+            self.log.info('Applying default configuration...')
+        else:
+            print(f'> Applying configuration from "{self.configuration.file_path}"...')
+            self.log.info(f'Applying configuration from "{self.configuration.file_path}"...')
 
         # Clear all qudi modules
         self.module_manager.clear()
@@ -214,18 +261,17 @@ class Qudi(QtCore.QObject):
         self._add_extensions_to_path()
 
         # Configure qudi modules
-        for base in ('gui', 'logic', 'hardware'):
+        for base in ['hardware', 'logic', 'gui']:
             # Create ManagedModule instance by adding each module to ModuleManager
-            for module_name, module_cfg in self.configuration.module_config[base].items():
+            for module_name, module_cfg in self.configuration[base].items():
                 try:
                     self.module_manager.add_module(name=module_name,
                                                    base=base,
                                                    configuration=module_cfg)
                 except:
                     self.module_manager.remove_module(module_name, ignore_missing=True)
-                    self.log.exception(
-                        f'Unable to create ManagedModule instance for module "{base}.{module_name}"'
-                    )
+                    self.log.exception(f'Unable to create ManagedModule instance for {base} '
+                                       f'module "{module_name}"')
 
         print('> Qudi configuration complete!')
         self.log.info('Qudi configuration complete!')
@@ -233,14 +279,19 @@ class Qudi(QtCore.QObject):
     def _start_gui(self):
         if self.no_gui:
             return
-        self.gui = Gui(qudi_instance=self, stylesheet_path=self.configuration.stylesheet)
-        self.gui.activate_main_gui()
+        self.gui = Gui(qudi_instance=self, stylesheet_path=self.configuration['stylesheet'])
+        if not self.configuration['hide_manager_window']:
+            self.gui.activate_main_gui()
 
     def _start_startup_modules(self):
-        for module in self.configuration.startup_modules:
+        for module in self.configuration['startup_modules']:
             print(f'> Loading startup module: {module}')
             self.log.info(f'Loading startup module: {module}')
-            self.module_manager.activate_module(module)
+            # Do not crash if a module can not be started
+            try:
+                self.module_manager.activate_module(module)
+            except:
+                self.log.exception(f'Unable to activate autostart module "{module}":')
 
     def run(self):
         """
@@ -249,23 +300,6 @@ class Qudi(QtCore.QObject):
             if self._is_running:
                 raise RuntimeError('Qudi is already running!')
 
-            # Disable pyqtgraph "application exit workarounds" because they cause errors on exit
-            try:
-                import pyqtgraph
-                pyqtgraph.setConfigOption('exitCleanup', False)
-            except ImportError:
-                pass
-
-            # Enable stack trace output for SIGSEGV, SIGFPE, SIGABRT, SIGBUS and SIGILL signals
-            # -> e.g. for segmentation faults
-            faulthandler.disable()
-            faulthandler.enable(all_threads=True)
-
-            # install logging facility and set logging level
-            init_record_model_handler(max_records=10000)
-            init_rotating_file_handler(path=self.log_dir)
-            set_log_level(DEBUG if self.debug_mode else INFO)
-
             # Notify startup
             startup_info = f'Starting qudi{" in debug mode..." if self.debug_mode else "..."}'
             self.log.info(startup_info)
@@ -355,6 +389,8 @@ class Qudi(QtCore.QObject):
             QtCore.QCoreApplication.instance().processEvents()
             self.log.info('Qudi shutting down...')
             print('> Qudi shutting down...')
+            self.watchdog.terminate()
+            self.watchdog = None
             self.log.info('Stopping module server(s)...')
             print('> Stopping module server(s)...')
             if self.remote_modules_server is not None:
@@ -385,6 +421,8 @@ class Qudi(QtCore.QObject):
             print('> Stopping remaining threads...')
             self.thread_manager.quit_all_threads()
             QtCore.QCoreApplication.instance().processEvents()
+            clear_handlers()
+            gc.collect()  # Explicit gc call to prevent Qt C++ extensions from using deleted Python objects
             if restart:
                 QtCore.QCoreApplication.exit(42)
             else:
