@@ -26,7 +26,7 @@ from abc import abstractmethod
 from uuid import uuid4
 from fysom import Fysom
 from PySide2 import QtCore, QtGui, QtWidgets
-from typing import Any, Mapping, Optional, Callable, Union
+from typing import Any, Mapping, Optional, Callable, Union, Dict
 
 from qudi.core.configoption import MissingOption
 from qudi.core.statusvariable import StatusVar
@@ -66,28 +66,6 @@ class ModuleStateMachine(Fysom, QtCore.QObject):
         # Initialise state machine:
         super().__init__(parent=parent, cfg=fsm_cfg, **kwargs)
 
-    def _build_event(self, event):
-        """
-        Overrides Fysom _build_event to wrap on_activate and on_deactivate to catch and log
-        exceptions.
-        @param str event: Event name to build the Fysom event for
-        @return function: The event handler used by Fysom for the given event
-        """
-        base_event = super()._build_event(event)
-        if event in ('activate', 'deactivate'):
-            noun = 'activation' if event == 'activate' else 'deactivation'
-
-            def wrap_event(*args, **kwargs):
-                try:
-                    base_event(*args, **kwargs)
-                except:
-                    self.parent().log.exception('Error during {0}'.format(noun))
-                    return False
-                return True
-
-            return wrap_event
-        return base_event
-
     def __call__(self) -> str:
         """
         Returns the current state.
@@ -181,8 +159,8 @@ class Base(QtCore.QObject, metaclass=ModuleMeta):
         self.__initialize_connectors()
 
         # Initialize module FSM
-        default_callbacks = {'on_before_activate': self.__activation_callback,
-                             'on_deactivate'     : self.__deactivation_callback}
+        default_callbacks = {'on_before_activate'  : self.__activation_callback,
+                             'on_before_deactivate': self.__deactivation_callback}
         default_callbacks.update(callbacks)
         self.module_state = ModuleStateMachine(parent=self, callbacks=default_callbacks)
         return
@@ -269,15 +247,30 @@ class Base(QtCore.QObject, metaclass=ModuleMeta):
         with a very good reason.
         """
         config = self._qudi_main.configuration
-        data_root = config.default_data_dir
+        data_root = config['default_data_dir']
         if data_root is None:
             data_root = get_default_data_dir()
-        if config.daily_data_dirs or config.daily_data_dirs is None:
+        if config['daily_data_dirs']:
             data_dir = os.path.join(get_daily_directory(root=data_root), self.module_name)
         else:
             data_dir = os.path.join(data_root, self.module_name)
         return data_dir
 
+    @property
+    def module_status_variables(self) -> Dict[str, Any]:
+        variables = dict()
+        try:
+            for attr_name, var in self._meta['status_variables'].items():
+                if hasattr(self, attr_name):
+                    value = getattr(self, attr_name)
+                    if not isinstance(value, StatusVar):
+                        if var.representer_function is not None:
+                            value = var.representer_function(self, value)
+                        variables[var.name] = value
+        except:
+            self.log.exception('Error while collecting status variables:')
+        return variables
+
     @property
     def _qudi_main(self) -> Any:
         qudi_main = self.__qudi_main_weakref()
@@ -300,37 +293,42 @@ class Base(QtCore.QObject, metaclass=ModuleMeta):
         """
         return self._threaded
 
-    def __activation_callback(self, event=None) -> None:
+    def __activation_callback(self, event=None) -> bool:
         """ Restore status variables before activation and invoke on_activate method.
         """
-        self._load_status_variables()
-        self.on_activate()
+        try:
+            self._load_status_variables()
+            self.on_activate()
+        except:
+            self.log.exception('Exception during activation:')
+            return False
+        return True
 
-    def __deactivation_callback(self, event=None) -> None:
+    def __deactivation_callback(self, event=None) -> bool:
         """ Invoke on_deactivate method and save status variables afterwards even if deactivation
         fails.
         """
         try:
             self.on_deactivate()
+        except:
+            self.log.exception('Exception during deactivation:')
         finally:
             # save status variables even if deactivation failed
             self._dump_status_variables()
+        return True
 
     def _load_status_variables(self) -> None:
         """ Load status variables from app data directory on disc.
         """
         # Load status variables from app data directory
-        class_name = self.__class__.__name__
-        name = self.module_name
-        base = self.module_base
-        file_path = get_module_app_data_path(class_name, base, name)
+        file_path = get_module_app_data_path(self.__class__.__name__,
+                                             self.module_base,
+                                             self.module_name)
         try:
             variables = yaml_load(file_path, ignore_missing=True)
         except:
             variables = dict()
-            self.log.exception(
-                f'Failed to load status variables for module "{class_name}_{base}_{name}".'
-            )
+            self.log.exception('Failed to load status variables:')
 
         # Set instance attributes according to StatusVar meta objects
         try:
@@ -340,9 +338,7 @@ class Base(QtCore.QObject, metaclass=ModuleMeta):
                     value = var.constructor_function(self, value)
                 setattr(self, attr_name, value)
         except:
-            self.log.exception(
-                f'Error while settings status variables in module "{class_name}_{base}_{name}".'
-            )
+            self.log.exception('Error while settings status variables:')
 
     def _dump_status_variables(self) -> None:
         """ Dump status variables to app data directory on disc.
@@ -350,33 +346,17 @@ class Base(QtCore.QObject, metaclass=ModuleMeta):
         This method can also be used to manually dump status variables independent of the automatic
         dump during module deactivation.
         """
-        class_name = self.__class__.__name__
-        name = self.module_name
-        base = self.module_base
-        file_path = get_module_app_data_path(class_name, base, name)
+        file_path = get_module_app_data_path(self.__class__.__name__,
+                                             self.module_base,
+                                             self.module_name)
         # collect StatusVar values into dictionary
-        variables = dict()
-        try:
-            for attr_name, var in self._meta['status_variables'].items():
-                if hasattr(self, attr_name):
-                    value = getattr(self, attr_name)
-                    if not isinstance(value, StatusVar):
-                        if var.representer_function is not None:
-                            value = var.representer_function(self, value)
-                        variables[var.name] = value
-        except:
-            self.log.exception(
-                f'Error while collecting status variables from module "{class_name}_{base}_{name}".'
-            )
-
+        variables = self.module_status_variables
         # Save to file if any StatusVars have been found
         if variables:
             try:
                 yaml_dump(file_path, variables)
             except:
-                self.log.exception(
-                    f'Failed to save status variables for module "{class_name}.{base}.{name}".'
-                )
+                self.log.exception('Failed to save status variables:')
 
     def _send_balloon_message(self, title: str, message: str, time: Optional[float] = None,
                               icon: Optional[QtGui.QIcon] = None) -> None:
@@ -439,15 +419,13 @@ class Base(QtCore.QObject, metaclass=ModuleMeta):
     def on_activate(self) -> None:
         """ Method called when module is activated. Must be implemented by actual qudi module.
         """
-        raise NotImplementedError(f'Please implement and specify the activation method for '
-                                  f'{self.__class__.__name__}.')
+        raise NotImplementedError('Please implement and specify the activation method.')
 
     @abstractmethod
     def on_deactivate(self) -> None:
         """ Method called when module is deactivated. Must be implemented by actual qudi module.
         """
-        raise NotImplementedError(f'Please implement and specify the deactivation method '
-                                  f'{self.__class__.__name__}.')
+        raise NotImplementedError('Please implement and specify the deactivation method.')
 
 
 class LogicBase(Base):
