@@ -351,25 +351,25 @@ class Gaussian2D(FitModelBase):
     @estimator('Peak')
     def estimate_peak(self, data, xy):
         # ToDo: Not properly implemented, yet
-        x_range = abs(max(xy[0]) - min(xy[0]))
-        y_range = abs(max(xy[1]) - min(xy[1]))
+        x_range = abs(np.max(xy[0]) - np.min(xy[0]))
+        y_range = abs(np.max(xy[1]) - np.min(xy[1]))
 
-        amplitude = max(data)
-        center_x = x_range / 2 + min(xy[0])
-        center_y = y_range / 2 + min(xy[1])
+        amplitude = np.max(data)
+        center_x = x_range / 2 + np.min(xy[0])
+        center_y = y_range / 2 + np.min(xy[1])
         sigma_x = x_range / 10
         sigma_y = y_range / 10
         theta = 0
 
         estimate = self.make_params()
-        estimate['offset'].set(value=np.mean(data), min=-np.inf, max=max(data))
+        estimate['offset'].set(value=np.mean(data), min=-np.inf, max=np.max(data))
         estimate['amplitude'].set(value=amplitude, min=0, max=amplitude * 2)
         estimate['center_x'].set(value=center_x,
                                  min=np.min(xy[0]) - x_range / 2,
                                  max=np.max(xy[0]) + x_range / 2)
         estimate['center_y'].set(value=center_y,
-                                 min=np.min(xy[1]) - x_range / 2,
-                                 max=np.max(xy[1]) + x_range / 2)
+                                 min=np.min(xy[1]) - y_range / 2,
+                                 max=np.max(xy[1]) + y_range / 2)
         estimate['sigma_x'].set(value=sigma_x, min=x_range / (xy[0].shape[0] - 1), max=x_range)
         estimate['sigma_y'].set(value=sigma_y, min=y_range / (xy[0].shape[1] - 1), max=y_range)
         estimate['theta'].set(value=theta, min=-np.pi, max=np.pi)
