@@ -28,6 +28,7 @@ from PySide2 import QtCore, QtWidgets, QtGui
 from qudi.util.datafitting import FitContainer, FitConfigurationsModel, FitConfiguration
 from qudi.util.paths import get_artwork_dir
 from qudi.util.widgets.scientific_spinbox import ScienDSpinBox
+from qudi.util.widgets.separator_lines import HorizontalLine
 
 
 class FitWidget(QtWidgets.QWidget):
@@ -121,7 +122,7 @@ class FitConfigurationWidget(QtWidgets.QWidget):
         self.setLayout(main_layout)
         main_layout.setContentsMargins(0, 0, 0, 0)
 
-        icon_dir = os.path.join(get_artwork_dir(), 'icons', 'oxygen', '64x64')
+        icon_dir = os.path.join(get_artwork_dir(), 'icons')
 
         # Create new fit config editor elements
         self.model_combobox = QtWidgets.QComboBox()
@@ -130,7 +131,7 @@ class FitConfigurationWidget(QtWidgets.QWidget):
         self.name_lineedit = QtWidgets.QLineEdit()
         self.add_config_toolbutton = QtWidgets.QToolButton()
         self.add_config_toolbutton.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
-        self.add_config_toolbutton.setIcon(QtGui.QIcon(os.path.join(icon_dir, 'add-icon.png')))
+        self.add_config_toolbutton.setIcon(QtGui.QIcon(os.path.join(icon_dir, 'list-add')))
         hlayout = QtWidgets.QHBoxLayout()
         hlayout.addWidget(self.model_combobox)
         hlayout.addWidget(self.name_lineedit)
@@ -249,12 +250,12 @@ class _FitConfigPanel(QtWidgets.QWidget):
         groupbox.setLayout(main_layout)
 
         # add remove button
-        icon_dir = os.path.join(get_artwork_dir(), 'icons', 'oxygen', '64x64')
+        icon_dir = os.path.join(get_artwork_dir(), 'icons')
         self._name = fit_config.name
         self.remove_config_toolbutton = QtWidgets.QToolButton()
         self.remove_config_toolbutton.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
         self.remove_config_toolbutton.setIcon(
-            QtGui.QIcon(os.path.join(icon_dir, 'remove-icon.png'))
+            QtGui.QIcon(os.path.join(icon_dir, 'list-remove'))
         )
         self.remove_config_toolbutton.clicked.connect(
             lambda: self.sigConfigurationRemovedClicked.emit(self._name)
@@ -274,9 +275,7 @@ class _FitConfigPanel(QtWidgets.QWidget):
         hlayout.addWidget(self.remove_config_toolbutton)
 
         # add horizontal line
-        hline = QtWidgets.QFrame()
-        hline.setFrameShape(QtWidgets.QFrame.HLine)
-        main_layout.addWidget(hline)
+        main_layout.addWidget(HorizontalLine())
 
         # add parameters
         param_layout = QtWidgets.QGridLayout()
